// Crimson Fields -- a game of tactical warfare
// Copyright (C) 2000-2001 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

////////////////////////////////////////////////////////////////////////
// initwindow.cpp
//
// History:
//  23-03-2001 - created
//  29-05-2001 - moved CreateFilesList() to fileio.h
////////////////////////////////////////////////////////////////////////

#include <string.h>
#include "SDL_endian.h"

#include "initwindow.h"
#include "fileio.h"
#include "misc.h"
#include "game.h"

#define ICON_QUIT_X	185
#define ICON_QUIT_Y	57
#define ICON_QUIT_W	7
#define ICON_QUIT_H	7

#define IW_WIDGET_LIST  	0
#define IW_WIDGET_START_NEW	1
#define IW_WIDGET_START_SAVED	2
#define IW_WIDGET_LOAD_SAVED	3
#define IW_WIDGET_LOAD_NEW	4
#define IW_WIDGET_PBEM		5
#define IW_WIDGET_AI		6
#define IW_WIDGET_VIDEO_OPTIONS 7


struct initLevInfo {    // used as user_data for the LevListWidget
  bool password;
  bool singleplayer;
  string *full_path;
};

////////////////////////////////////////////////////////////////////////
// NAME       : LevListWidget::PrintItem
// DESCRIPTION: Print a level name. If the map is protected by a
//              password print it in red colour, otherwise in green.
//              Put the default number of players behind the title in
//              brackets.
// PARAMETERS : item - node to print
//              dest - destination surface
//              x    - left edge of printing area
//              y    - top edge of printing area
//              clip - clipping rectangle
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void LevListWidget::PrintItem( const TLWNode *item, Surface *dest,
                               short x, short y, const Rect &clip ) const {
  Color col( CF_COLOR_WHITE );
  string name( item->name, strlen(item->name)-4 ); // remove .lev or .sav suffix

  if ( item->user_data ) {
    initLevInfo *info = (initLevInfo *)item->user_data;
    if ( info->password ) col = surface->GetFGPen();
    name += ' ';
    name += '(';
    name += (info->singleplayer ? '1' : '2');
    name += ')';
  }

  Rect oldclip;
  dest->GetClipRect( oldclip );
  dest->SetClipRect( clip );
  font->Write( name.c_str(), dest, x, y, col );
  dest->SetClipRect( oldclip );
}


////////////////////////////////////////////////////////////////////////
// NAME       : InitWindow::InitWindow
// DESCRIPTION: Create the main menu window. The player(s) can select
//              the level they want to play and set other options.
// PARAMETERS : view  - view to attach the window to
//              title - pointer to title window; must be closed when this
//                      window is closed
// RETURNS    : -
//
// HISTORY
//   04-04-2001 - quit button activated by pressing ESCAPE
//   19-08-2001 - added checkbox to play against AI
//   23-06-2002 - logo replaced by title screen; removed
//   14-08-2002 - added support for maps in $HOME
////////////////////////////////////////////////////////////////////////

InitWindow::InitWindow( View *view, Window *title ) : Window( WIN_CENTER, view ) {
  this->title = title;

  // calculate window dimensions
  SetSize( MIN(380, view->Width()), sfont->Height() * 15 + 10 );
  view->SetFGPen( Color( 0x00d87c00 ) );
  view->SetBGPen( Color( CF_COLOR_BLACK ) );

  const string home_lev( get_home_levels_dir() );
  if ( !home_lev.empty() ) create_files_list( home_lev.c_str(), ".lev", levels );
  create_files_list( get_levels_dir().c_str(), ".lev", levels );
  CompleteFilesList( levels );
  create_files_list( get_save_dir().c_str(), ".sav", saves );
  CompleteFilesList( saves );

  // create widgets
  levwidget = new LevListWidget( IW_WIDGET_LIST, 5, 5,
                    w/2 - 10, sfont->Height() * 15,
                    &levels, -1, WIDGET_ARROWSCROLL, NULL, this );

  startwidget = new ButtonWidget( IW_WIDGET_START_NEW, w/2 + 5,
                levwidget->TopEdge() + levwidget->Height() - sfont->Height() - 8,
                w/2 - 10, sfont->Height() + 8, 's',
                WIDGET_DEFAULT, "Start", this );
  startwidget->SetHook( this );

  videowidget = new ButtonWidget( IW_WIDGET_VIDEO_OPTIONS,
                startwidget->LeftEdge(), levwidget->TopEdge(),
                startwidget->Width() - ICON_QUIT_W - 13, startwidget->Height(),
                'v', 0, "Video Options", this );
  videowidget->SetHook( this );

  listctrlwidget = new ButtonWidget( IW_WIDGET_LOAD_SAVED, startwidget->LeftEdge(),
                   videowidget->TopEdge() + videowidget->Height() + 5,
                   startwidget->Width(), videowidget->Height(), 'l', 0,
                   "Load saved game", this );
  listctrlwidget->SetHook( this );

  pbemwidget = new CheckboxWidget( IW_WIDGET_PBEM, listctrlwidget->LeftEdge() + 5,
                   listctrlwidget->TopEdge() + listctrlwidget->Height() + 10,
                   DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, false, 'e',
                   WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                   "Play By e-Mail", this );
  pbemwidget->SetHook( this );

  aiwidget = new CheckboxWidget( IW_WIDGET_AI, pbemwidget->LeftEdge(),
                   pbemwidget->TopEdge() + pbemwidget->Height() + 3,
                   DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, false, 'c',
                   WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                   "Play vs. Computer", this );
  aiwidget->SetHook( this );

  // the quit button
  ButtonWidget *btn = new ButtonWidget( GUI_QUIT, w - ICON_QUIT_W - 9,
                   videowidget->TopEdge(), ICON_QUIT_W + 4, ICON_QUIT_H + 4,
                   SDLK_ESCAPE, WIDGET_STYLE_GFX, NULL, this );
  btn->SetImage( view->GetSystemIcons(),
                   Rect( ICON_QUIT_X, ICON_QUIT_Y, ICON_QUIT_W, ICON_QUIT_H ),
                   Rect( ICON_QUIT_X, ICON_QUIT_Y, ICON_QUIT_W, ICON_QUIT_H ) );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : InitWindow::~InitWindow
// DESCRIPTION: Destroy window.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
//   17-06-2002 - free save games list, too
////////////////////////////////////////////////////////////////////////

InitWindow::~InitWindow( void ) {
  TLWNode *n;
  initLevInfo *li;

  while ( !levels.IsEmpty() ) {
    n = static_cast<TLWNode *>( levels.RemHead() );
    li = (initLevInfo *)n->user_data;
    if ( li ) {
      delete li->full_path;
      delete li;
    }
    delete [] n->name;
    delete n;
  }

  while ( !saves.IsEmpty() ) {
    n = static_cast<TLWNode *>( saves.RemHead() );
    li = (initLevInfo *)n->user_data;
    if ( li ) {
      delete li->full_path;
      delete li;
    }
    delete [] n->name;
    delete n;
  }
}

////////////////////////////////////////////////////////////////////////
// NAME       : InitWindow::Draw
// DESCRIPTION: Draw the main window.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void InitWindow::Draw( void ) {
  Window::Draw();
  DrawBox( Rect( listctrlwidget->LeftEdge(),
                 pbemwidget->TopEdge() - 5,
                 listctrlwidget->Width(),
                 startwidget->TopEdge() - pbemwidget->TopEdge() - 5),
           BOX_CARVED );
}

////////////////////////////////////////////////////////////////////////
// NAME       : InitWindow::Activate
// DESCRIPTION: Handle activation of widgets in the window.
// PARAMETERS : button - calling widget
//              win    - pointer to active window
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status InitWindow::Activate( ButtonWidget *button, Window *win ) {

  switch ( button->ID() ) {
  case IW_WIDGET_START_NEW:
  case IW_WIDGET_START_SAVED: {
    TLWNode *n = static_cast<TLWNode *>( levwidget->Selected() );
    if ( !n ) new NoteWindow( "Error!", "No level selected!", WIN_CLOSE_ESC, view );
    else {
      unsigned short game_flags = 0;
      string *lname = ((initLevInfo *)n->user_data)->full_path;
      if ( button->ID() == IW_WIDGET_START_NEW ) {
        if ( pbemwidget->Clicked() &&
             !pbemwidget->Disabled() ) game_flags |= GI_PBEM;
        if ( aiwidget->Clicked() &&
             !aiwidget->Disabled() ) game_flags |= GI_AI;
      }

      Gam = new Game( game_flags, view );
      if ( Gam->Load( lname->c_str() ) ) {
        NoteWindow *nw = new NoteWindow( "Error!", "Failed to load mission!", 0, view );
        nw->SetButtonID( 0, GUI_RESTART );
      } else if ( !Gam->GetLevelPassword() ||
                  Gam->CheckPassword( "Access restricted", "Enter access code",
                                      Gam->GetLevelPassword(), false ) ) {
        view->CloseWindow( this );
        view->Refresh();
        Gam->StartTurn();         // let the games begin...
      } else {
#ifndef arm      
        delete Gam;
#endif
        Gam = NULL;
        new NoteWindow( "Access denied!", "Invalid access code for this level.",
                        WIN_CLOSE_ESC, view );
      }
    }
    break; }
  case IW_WIDGET_LOAD_NEW:
    startwidget->SetID( IW_WIDGET_START_NEW );
    listctrlwidget->SetID( IW_WIDGET_LOAD_SAVED );
    listctrlwidget->SetTitle( "Load saved game" );
    listctrlwidget->Draw();
    levwidget->SwitchList( &levels, -1 );
    pbemwidget->Enable();
    pbemwidget->Draw();
    aiwidget->Enable();
    aiwidget->Draw();
    Show();
    break;
  case IW_WIDGET_LOAD_SAVED:
    startwidget->SetID( IW_WIDGET_START_SAVED );
    listctrlwidget->SetID( IW_WIDGET_LOAD_NEW );
    listctrlwidget->SetTitle( "Launch new game" );
    listctrlwidget->Draw();
    levwidget->SwitchList( &saves, -1 );
    pbemwidget->Disable();
    pbemwidget->Draw();
    aiwidget->Disable();
    aiwidget->Draw();
    Show();
    break;
  case IW_WIDGET_PBEM:
    if ( pbemwidget->Clicked() ) aiwidget->Disable();
    else aiwidget->Enable();
    aiwidget->Draw();
    Show();
    break;
  case IW_WIDGET_AI:
    if ( aiwidget->Clicked() ) pbemwidget->Disable();
    else pbemwidget->Enable();
    pbemwidget->Draw();
    Show();
    break;
  case IW_WIDGET_VIDEO_OPTIONS:
    new VideoOptionsWindow( view );
    break;
  }

  return GUI_OK;
}

////////////////////////////////////////////////////////////////////////
// NAME       : InitWindow::CompleteFilesList
// DESCRIPTION: For the levels list display we need some information
//              about the maps (password protection, 1 or 2 players).
//              This information is attached to the list items.
// PARAMETERS : list - list of files created using create_files_list()
// RETURNS    : GUI status
//
// HISTORY
//   14-08-2002 - added support for maps in  $HOME
//   21-08-2002 - remove invalid files from the list
//              - close files after use
////////////////////////////////////////////////////////////////////////

void InitWindow::CompleteFilesList( List &list ) const {
  initLevInfo *info;

  for ( TLWNode *n = static_cast<TLWNode *>(list.Head()), *next; n; n = next ) {
    next = static_cast<TLWNode *>(n->Next());
    string *fname = (string *)n->user_data;
    bool ok = false;

    SDL_RWops *file = SDL_RWFromFile( fname->c_str(), "rb" );
    if ( file ) {
      unsigned long id;
      unsigned short flags;
      unsigned char ver;

      id = SDL_ReadLE32( file );
      if ( id == FID_MISSION ) {
        SDL_RWread( file, &ver, 1, 1 );
        if ( ver == FILE_VERSION ) {
          flags = SDL_ReadLE16( file );
          info = new initLevInfo;
          info->password = ((flags & GI_PASSWORD) != 0);
          info->singleplayer = ((flags & GI_AI) != 0);
          info->full_path = fname;
          n->user_data = info;
          ok = true;
        }
      }
      SDL_RWclose( file );
    }

    if ( !ok ) {
      n->Remove();
      delete (string *)n->user_data;
      delete [] n->name;
      delete n;
    }
  }
}

////////////////////////////////////////////////////////////////////////
// NAME       : TitleWindow::TitleWindow
// DESCRIPTION: Create a mostly invisible window containing the title
//              screen image.
// PARAMETERS : view - view to attach the window to
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

TitleWindow::TitleWindow( View *view ) : Window( WIN_CENTER, view ) {
  string tname = get_data_dir();
  append_path_delim( tname );
  tname.append( CF_TITLE_SCREEN );

  if ( LoadBMP( tname.c_str() ) ) SetSize( 0, 0, 0, 0 );
  else {
    DisplayFormat();
    Center( *view ); 

    if ( *view >= *this ) {
      Show();
      EventLoop();
    }
  }
}

////////////////////////////////////////////////////////////////////////
// NAME       : TitleWindow::HandleEvent
// DESCRIPTION: Close (or rather, return control to main) when a key or
//              a mouse button is pressed.
// PARAMETERS : event - event received by the event handler
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status TitleWindow::HandleEvent( const SDL_Event &event ) {
  GUI_Status rc = GUI_OK;
  if ( (event.type == SDL_KEYDOWN) || (event.type == SDL_MOUSEBUTTONDOWN) )
    rc = GUI_CLOSE;
  return rc;
}

////////////////////////////////////////////////////////////////////////
// NAME       : TitleWindow::Show
// DESCRIPTION: Show window if it fits on the screen
// PARAMETERs : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void TitleWindow::Show( void ) {
  if ( *view >= *this ) Window::Show();
}

#define VOW_WIDGET_MODES	0
#define VOW_WIDGET_OK		1
#define VOW_WIDGET_BACK		GUI_CLOSE
#define VOW_WIDGET_FULLSCREEN	3

////////////////////////////////////////////////////////////////////////
// NAME       : VideoOptionsWindow::VideoOptionsWindow
// DESCRIPTION: Show the video options menu.
// PARAMETERS : view - view to attach the window to
// RETURNS    : -
//
// HISTORY
//   24-10-2001 - don't show modes smaller than 640x480
//   21-08-2002 - fix size of allocated char array (+1)
////////////////////////////////////////////////////////////////////////

VideoOptionsWindow::VideoOptionsWindow( View *view ) : Window( WIN_CENTER, view ) {
  static SDL_Rect std_sizes[] = { { 1280, 1024 }, { 1024, 768 },
                                  { 800, 600}, { 640, 480 } };
  SDL_Rect *std_modes[5];
  short current = -1;

  // calculate window dimensions
  SetSize( lfont->Width() * 14 + 30, sfont->Height() * 12 + lfont->Height() + 45 );

  // create modes list
  SDL_Rect **sdl_modes = SDL_ListModes( NULL, SDL_FULLSCREEN );

  // if any mode is ok offer some standard ones
  if ( sdl_modes == (SDL_Rect **)-1 ) {
    sdl_modes = std_modes;
    std_modes[0] = &std_sizes[0];
    std_modes[1] = &std_sizes[1];
    std_modes[2] = &std_sizes[2];
    std_modes[3] = &std_sizes[3];
    std_modes[4] = NULL;
  }
 
  if ( sdl_modes != NULL ) {
    char buf[12];

    for ( int i = 0; sdl_modes[i]; ++i ) {
      if ( (sdl_modes[i]->w >= MIN_XRES) && (sdl_modes[i]->h >= MIN_YRES) ) {
        TLWNode *n = new TLWNode;
        sprintf( buf, "%d x %d", sdl_modes[i]->w, sdl_modes[i]->h );

        n->name = new char[ strlen(buf)+1 ];
        n->user_data = sdl_modes[i];
        strcpy( n->name, buf );
        modes.AddTail( n );

        if ( (sdl_modes[i]->w == view->Width()) &&
             (sdl_modes[i]->h == view->Height()) ) current = i;
      }
    }
  }

  // create widgets
  modewidget = new TextListWidget( VOW_WIDGET_MODES, 10, lfont->Height() + 20,
                    w - 20, sfont->Height() * 10, &modes, current,
                    WIDGET_ARROWSCROLL|WIDGET_ALIGN_CENTER, NULL, this );

  ButtonWidget *ok = new ButtonWidget( VOW_WIDGET_OK, 1,
                     Height() - sfont->Height() - 9, (Width() - 2) / 2,
                     sfont->Height() + 8, 's', 0, "Set", this );
  ok->SetHook( this );

  new ButtonWidget( VOW_WIDGET_BACK, 1 + (Width() - 2) / 2, Height() - sfont->Height() - 9,
                    (Width() - 2) / 2, sfont->Height() + 8, 'c',
                    WIDGET_DEFAULT, "Close", this );

  fswidget = new CheckboxWidget( VOW_WIDGET_FULLSCREEN, modewidget->LeftEdge() + 10,
                   modewidget->TopEdge() + modewidget->Height() + 10,
                   DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, view->IsFullScreen(), 'f',
                   WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                   "Fullscreen", this );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : VideoOptionsWindow::~VideoOptionsWindow
// DESCRIPTION: Destroy window.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

VideoOptionsWindow::~VideoOptionsWindow( void ) {
  while ( !modes.IsEmpty() ) {
    TLWNode *n = static_cast<TLWNode *>( modes.RemHead() );
    delete [] n->name;
    delete n;
  }
}

////////////////////////////////////////////////////////////////////////
// NAME       : VideoOptionsWindow::Draw
// DESCRIPTION: Draw the video options window.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void VideoOptionsWindow::Draw( void ) {
  Window::Draw();

  const char *title = "Video Options";

  lfont->Write( title, this, (w - lfont->TextWidth(title)) / 2 + 3,
                             11, view->GetBGPen() );
  lfont->Write( title, this, (w - lfont->TextWidth(title)) / 2,
                             8, view->GetFGPen() );
}

////////////////////////////////////////////////////////////////////////
// NAME       : VideoOptionsWindow::Activate
// DESCRIPTION: When the user pushes the 'Use' button, switch to the
//              selected video mode.
// PARAMETERS : button - calling widget
//              win    - pointer to active window
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status VideoOptionsWindow::Activate( ButtonWidget *button, Window *win ) {
  TLWNode *mode = static_cast<TLWNode *>( modewidget->Selected() );

  if ( mode ) {
    bool fs = fswidget->Clicked();
    unsigned long mode_flags = SDL_HWSURFACE|(fs ? SDL_FULLSCREEN : 0);

    // if selected mode is the same as current mode only check for fullscreen
    // dimensions of the selected mode are available in the user_data field
    SDL_Rect *dim = (SDL_Rect *)mode->user_data;
    if ( (dim->w == view->Width()) && (dim->h == view->Height()) ) {
      if ( fs != view->IsFullScreen() ) view->ToggleFullScreen();
    } else {
      view->SetVideoMode( dim->w, dim->h, DISPLAY_BPP, mode_flags );
    }
  }

  return GUI_OK;
}

