// CoMET - The Crimson Fields Map Editing Tool
// Copyright (C) 2002-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

////////////////////////////////////////////////////////////////////////
// extwindow2.cpp
//
// History:
//  06-12-2002 - created
///////////////////////////////////////////////////////////////////////

#include <stdlib.h>

#include "extwindow2.h"

////////////////////////////////////////////////////////////////////////
// NAME       : NewMissionWindow::NewMissionWindow
// DESCRIPTION: This window pops up when the user wants to create a new
//              mission. He can adjust a few basic settings like map
//              size or tileset and create the mission.
// PARAMETERS : view - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

NewMissionWindow::NewMissionWindow( View *view ) : Window(WIN_CENTER, view) {

  // calculate window dimensions
  Font *f = SmallFont();
  unsigned short w = MIN( f->Width() * 35 + 54, view->Width() - 20 );
  unsigned short h = f->Height() * 3 + 40;
  unsigned short xoff = f->Width() * 11 + 14, yoff = 5;

  SetSize( w, h );

  m_width = new NumberWidget( 1, xoff, yoff, f->Width() * 4 + 10,
            f->Height() + 6, 'w', "40", 3, WIDGET_ALIGN_LEFT,
            "Map Width :", this );

  tileset = new StringWidget( 3, xoff + m_width->w + 10 + f->Width() * 9,
            yoff, f->Width() * 11 + 10, m_width->h, 't', "default", 10,
            WIDGET_DISABLED|WIDGET_ALIGN_LEFT, "Tile Set:", this );

  yoff += m_width->Height() + 5;

  m_height = new NumberWidget( 2, xoff, yoff, m_width->w, m_width->h,
            'h', "40", 3, WIDGET_ALIGN_LEFT, "Map Height:", this );

  unitset = new StringWidget( 4, tileset->x, yoff, tileset->w, tileset->h,
            'u', "default", 10, WIDGET_DISABLED|WIDGET_ALIGN_LEFT,
            "Unit Set:", this );

  yoff = h - f->Height() - 15;
  ok = new ButtonWidget( B_ID_NMW_CREATE, 10, yoff, f->Width() * 10,
       f->Height() + 10, 'o', 0, "OK", this );

  new ButtonWidget( GUI_CLOSE, w - 10 - ok->w, yoff, ok->w, ok->h,
                    'c', 0, "Cancel", this );
  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : NewMissionWindow::GetMapSize
// DESCRIPTION: Get the current slider settings.
// PARAMETERS : -
// RETURNS    : selected size as a Point
//
// HISTORY
////////////////////////////////////////////////////////////////////////

Point NewMissionWindow::GetMapSize( void ) const {
  Point s( 0, 0 );
  const char *str = m_width->String();
  if ( str ) s.x = atoi(str);

  str = m_height->String();
  if ( str ) s.y = atoi(str);

  if ( s.x < 20 ) s.x = 20;
  else if ( s.x > 200 ) s.x = 200;

  if ( s.y < 20 ) s.y = 20;
  else if ( s.y > 200 ) s.y = 200;

  return s;
}


////////////////////////////////////////////////////////////////////////
// NAME       : EdBuildingWindow::EdBuildingWindow
// DESCRIPTION: This window contains the settings for a building.
// PARAMETERS : b     - building to be modified
//              first - first unit currently on the map
//              view  - pointer to the window's view
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

EdBuildingWindow::EdBuildingWindow( Building &bld, Unit *first, View *view ) :
                  Window(WIN_CENTER, view), b(bld), unit(first) { 
  // calculate dimensions
  SetSize( MIN(sfont->Width() * 50, view->Width()),
           MIN(sfont->Height() * 12, view->Height()) );

  unsigned short xoff, yoff = 5, wdh = sfont->Height() + 10;

  b_name = new StringWidget( B_ID_PRV_EBW_NAME, 5, yoff, w/2 - 10, wdh,
                             'n', b.Name(), 19, 0, NULL, this );

  static const char *labels[] = { "One", "Two", "Neutral", 0 };
  xoff = w/2 + 10 + sfont->TextWidth("Player");
  b_player = new CycleWidget( B_ID_PRV_EBW_PLAYER, xoff, yoff, w - 5 - xoff, wdh,
                              'p', 0, "Player", b.Owner(), labels, this );
  yoff += 5 + wdh;

  ButtonWidget *btn;
  btn = new CheckboxWidget( B_ID_PRV_EBW_WORKSHOP, 5, yoff, DEFAULT_CBW_SIZE,
                            DEFAULT_CBW_SIZE, b.IsWorkshop(), 'w',
                            WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                            "Workshop", this );
  btn->SetHook( this );

  btn = new CheckboxWidget( B_ID_PRV_EBW_FACTORY, 5, yoff + DEFAULT_CBW_SIZE + 5,
                            DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, b.IsFactory(), 'f',
                            WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
                            "Factory", this );
  btn->SetHook( this );


  btn = new ButtonWidget( B_ID_PRV_EBW_OK, 1, h - 1 - wdh, w - 2, wdh, 'o',
                          WIDGET_DEFAULT, "OK", this ),
  btn->SetHook( this );

  Draw();
  Show();
}

////////////////////////////////////////////////////////////////////////
// NAME       : EdBuildingWindow::Activate
// DESCRIPTION: React to user actions.
// PARAMETERS : button - activated button widget
//              win    - window containing the button
// RETURNS    : GUI status
//
// HISTORY
////////////////////////////////////////////////////////////////////////

GUI_Status EdBuildingWindow::Activate( ButtonWidget *button, Window *win ) {

  switch ( button->ID() ) {
  case B_ID_PRV_EBW_WORKSHOP:
    if ( button->Clicked() ) b.SetFlags( BLD_WORKSHOP );
    else b.UnsetFlags( BLD_WORKSHOP );
    break;
  case B_ID_PRV_EBW_FACTORY:
    if ( button->Clicked() ) b.SetFlags( BLD_FACTORY );
    else b.UnsetFlags( BLD_FACTORY );
    break;
  case B_ID_PRV_EBW_OK:
    view->CloseWindow( win );
    b.SetOwner( b_player->GetValue() );
    b.SetName( b_name->String() );

    while ( unit ) {
      if ( unit->Position() == b.Position() ) b.SetOwner( b.Owner() );
      unit = static_cast<Unit *>(unit->Next());
    }
    break;
  }

  return GUI_OK;
}

